%%%============================================================================
%%% Time-stamp: <2007-09-02 13:19:34 j>
%%%
%%% File    : eburg.hrl
%%% Purpose : Record definitions.
%%% Author  : j@crunchy.svirfneblin.org
%%% License : BSD
%%% Created : 2007-07-11 14:45:31 j
%%%
%%%============================================================================

%% The state record. Contains the internal representation of the input
%% specification plus some derived data.
-record(eburg,   {in_file,  % string
                  out_file, % string
                  module,   % string
                  ops,      % [atom]
                  nts,      % [atom]
                  arities,  % [{atom, int}]
                  nt_nums,  % [{atom, int}]
                  start,    % atom
                  rules,    % [#rule]
                  crules,   % [{atom, [#rule]}]
                  oprules,  % [{atom, [#rule]}]
                  code}).   % string

%% Rules are annotated with rule (1 to ,number_of_rules, in the order in
%% which they appear in the input file) and line numbers (for error messages).
-record(rule,    {number,   % int
                  line,     % int
                  lhs,      % atom
                  rhs,      % #pattern
                  cost = 0, % int | string
                  action}). % string

%% Patterns are used to represent the right-hand sides (RHSs) of rules.
%% Chainrules and nullary operators have and empty list of kids.
-record(pattern, {op,       % atom
                  kids,     % [#pattern]
                  arity,    % int
                  depth}).  % int

%%% Local Variables:
%%% mode: erlang
%%% End:
