%%%============================================================================
%%% Time-stamp: <2007-09-03 14:58:40 j>
%%%
%%% File    : eburg_lexer.xrl
%%% Purpose : Scan and tokenize eburg specifications.
%%% Author  : j@crunchy.svirfneblin.org
%%% License : BSD
%%% Created : 2007-07-07 10:40:08 j
%%%
%%%============================================================================

Definitions.
LOWER     = [a-z]
UPPER     = [A-Z]
DIGIT     = [0-9]
ANY       = ({LOWER}|{UPPER}|{DIGIT}|_)
ID        = {LOWER}{ANY}*

COST      = \[[^]]*\]
ACTION    = :\s[^;]*;\n

WS        = [\000-\s]+
COMMENT   = \%.*
EOF       = \$end

Rules.
Term      : {token, {'Term', TokenLine}}.
Start     : {token, {'Start', TokenLine}}.
{ID}      : {token, {id, TokenLine, list_to_atom(TokenChars)}}.

{ACTION}  : {token, {action, TokenLine, strip(butlast(TokenChars))}}.
{COST}    : {token, case catch list_to_integer(strip(TokenChars)) of
                        {'EXIT', _} -> {funcall, TokenLine, strip(TokenChars)};
                        Int         -> {int, TokenLine, Int}
                    end}.

<-        : {token, {'<-', TokenLine}}.
\(        : {token, {'(', TokenLine}}.
\)        : {token, {')', TokenLine}}.
,         : {token, {',', TokenLine}}.
;\n       : {token, {';', TokenLine}}.

{WS}      : skip_token.
{COMMENT} : skip_token.
{EOF}     : {end_token, {'$end', TokenLine}}.


Erlang code.
-export([lex/1]).

%% lex(File: string) -> string
%% returns File with the eof token attached (kludge).
lex(File) -> string(File ++ "$end").

%% strip(ActionOrCost: string) -> string
%% returns ActionOrCost with :./[] removed.
strip(Str) -> string:strip(butlast(tl(Str))).

%% butlast(Xs: [term]) -> [term]
%% returns Xs without its last element.
butlast([])     -> [];
butlast([_X])   -> [];
butlast([X|Xs]) -> [X|butlast(Xs)].

%%% Local Variables:
%%% mode: erlang
%%% End:
