%%%============================================================================
%%% Time-stamp: <2007-09-03 14:58:52 j>
%%%
%%% File    : eburg_parser.yrl
%%% Purpose : Parse eburg specifications.
%%% Author  : j@crunchy.svirfneblin.org
%%% License : BSD
%%% Created : 2007-06-07 10:53:58 j
%%%
%%%============================================================================

Nonterminals spec term start ids rules rule cost trees tree.
Terminals 'Start' 'Term' id funcall int action '<-' '(' ')' ',' ';'.
Rootsymbol spec.
Endsymbol '$end'.

spec  -> term start rules         : #eburg{ops = '$1', start = '$2', rules = '$3'}.
spec  -> term rules               : #eburg{ops   = '$1', 
                                           start = (hd('$2'))#rule.lhs,
                                           rules = '$2'}.
term  -> 'Term' ids ';'           : '$2'.
start -> 'Start' id ';'           : unwrap('$2').
ids   -> id                       : [unwrap('$1')].
ids   -> id ids                   : [unwrap('$1')|'$2'].
rules -> rule                     : ['$1'].
rules -> rule rules               : ['$1'|'$2'].
rule  -> id '<-' tree action      : #rule{line   = line('$1'),
                                          lhs    = unwrap('$1'),
                                          rhs    = '$3',
                                          action = unwrap('$4')}.
rule  -> id '<-' tree cost action : #rule{line   = line('$1'),
                                          lhs    = unwrap('$1'),
                                          rhs    = '$3',
                                          cost   = '$4',
                                          action = unwrap('$5')}.
cost  -> int                      : unwrap('$1').
cost  -> funcall                  : unwrap('$1').
trees -> tree                     : ['$1'].
trees -> tree ',' trees           : ['$1'|'$3'].
tree  -> id                       : #pattern{op    = unwrap('$1'),
                                             kids  = [],
                                             arity = 0,
                                             depth = 0}.
tree  -> id '(' trees  ')'        : #pattern{op    = unwrap('$1'),
                                             kids  = '$3',
                                             arity = length('$3'),
                                             depth = depth('$3')}.

Erlang code.
-include("eburg.hrl").

unwrap({_,_,Content}) -> Content.
line({_,Line,_})      -> Line.

depth(#pattern{kids=[]})   -> 0;
depth(#pattern{kids=Kids}) -> 1 + depth(Kids);
depth([])                  -> 0;
depth(Lst=[_X|_Xs])        -> lists:max(lists:map(fun depth/1, Lst)).

%%% Local Variables:
%%% mode: erlang
%%% End:
